<?php
namespace Controller;

use Util\Utilities;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;

class ChangedFiles extends \Generated\Controller\ChangedFiles {
	public function save(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        $dataSet     = $request->getDataSet();
        $showMessage = $this->verifyShowMessage($dataSet);
        parent::save($request, $response, $showMessage);
    }

    public function delete(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        $dataSet     = $request->getDataSet();
        $showMessage = $this->verifyShowMessage($dataSet);
        parent::delete($request, $response, $showMessage);
    }

    private function verifyShowMessage($dataSet) {      
        $showMessage = true;

        foreach ($dataSet->getRows() as $row) {
            if(isset($row['isMessage']) && $row['isMessage'] === false) {
                $showMessage = false;
                break;
            }
        }

        return $showMessage;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $this->getFindFilterData($request);
        $filterData->setDataSourceName($this->dataSourceName);
        $dataSet = $this->dataSourceManager->findBy($filterData);
        $response->addDataSet($dataSet);
    }

    /**
     * @param DTO\Request\Filter $request
     * @return FilterCriteria
     */
    private function getFindFilterData(DTO\Request\Filter $request)
    {
        $filterData = $request->getFilterCriteria();
        $conditions = $filterData->getConditions();
        $taskId = null;
        $taskId = Utilities::getValueInFilterDataSourceByName($conditions, 'TASK');
        if ($taskId) {
            $mainTask = \Util\Utilities::getMainRelatedTaskByTaskId($taskId);
            foreach ($conditions as &$condition) {
                if ($condition['columnName'] == 'TASK') {
                    $condition['value'] = $mainTask ? $mainTask->getId() : $taskId;
                }
            }
            $newFilterData = new FilterCriteria($this->dataSourceName, $conditions);
        } else {
            $newFilterData = $filterData;
        }

        return $newFilterData;
    }
}