<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Repositories;


use Doctrine\DBAL\Connection;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class ProjectType extends \Doctrine\ORM\EntityRepository {
    
    public function getProjectTypeByProfile($organization,$profiles,$filters = array(),$hasProduct = true){

        $filterString = Utilities::createQueryFiltered($filters);
        $filterMeeting = Utilities::queryFilterTypeProjectMeeting('PT',true);
        $filterNotHasProduct = '';
        if(!$hasProduct){
            $filterNotHasProduct = " AND PT.EXTERNALTYPE NOT IN(:controlHasProduct)";
        }
        $query = <<<SQL
            SELECT * FROM (
                SELECT      PT.ID,
                                (SELECT RTRIM(XMLAGG(XMLELEMENT(E,PA.PROFILE_ID,',').EXTRACT('//text()')
                                   ORDER BY PA.ID).GETCLOBVAL(),',')
                                   FROM PROJECT_APPROVAL PA
                                   WHERE (PA.PROJECT_TYPE_ID = PT.ID
                                          AND PA.TYPE = 'A')) AS PROFILE,
                               PT.USEMERGEREQUEST,PT.EXTERNALTYPE, PT.ORGANIZATION, PT.LABEL,
                               PT.REQUIRES_APPROVAL,PT.ALLOWENDTASK,PT.USETIMEPLANNING, PT.STATUS
                  FROM PROJECT_TYPE PT
             LEFT JOIN PROJECT_APPROVAL PU ON(PU.PROJECT_TYPE_ID = PT.ID AND PU.TYPE = 'U')
                 WHERE (PROFILE_ID IS NULL OR PROFILE_ID IN ($profiles))
                   AND PT.ORGANIZATION = :organization
                   AND PT.STATUS <> 'I'
                   $filterMeeting
                   $filterNotHasProduct
              GROUP BY PT.ID, PT.USETIMEPLANNING,PT.USEMERGEREQUEST, PT.EXTERNALTYPE, PT.ORGANIZATION, PT.LABEL,
                       PT.REQUIRES_APPROVAL,PT.ALLOWENDTASK, PT.STATUS
              ORDER BY ID)
              WHERE 1 = 1
        $filterString
        ORDER BY LABEL
SQL;

        $params = array(
            'organization' => $organization,
            'controlHasProduct' => array(
                Entities\ProjectType::TYPE_PROJECT_CONTROL_RELEASE,
                Entities\ProjectType::TYPE_PROJECT_CONTROL_SUB_RELEASE,
                Entities\ProjectType::TYPE_PROJECT_CONTROL_ERROR
            )

        );
        $paramTypes = array(
            'controlHasProduct' => Connection::PARAM_STR_ARRAY
        );
        $connection = $this->getEntityManager()->getConnection();
        $projectTypes = $connection->fetchAll($query, $params, $paramTypes);
        foreach ($projectTypes as &$projectType) {
            $projectType['PROFILE'] = Utilities::setSelectMultiInQuery($projectType['PROFILE'], true);
        }
        return $projectTypes;
    }

    public function getProjectTypesForProjectFilter($organization){

        $filterMeeting = Utilities::queryFilterTypeProjectMeeting('PT',true);
        $query = <<<SQL
    SELECT      PT.ID,
			        (SELECT RTRIM(XMLAGG(XMLELEMENT(E,PA.PROFILE_ID,',').EXTRACT('//text()')
                       ORDER BY PA.ID).GETCLOBVAL(),',')
                       FROM PROJECT_APPROVAL PA
                       WHERE (PA.PROJECT_TYPE_ID = PT.ID
                              AND PA.TYPE = 'A')) AS PROFILE,
			       PT.USEMERGEREQUEST,PT.EXTERNALTYPE, PT.ORGANIZATION, PT.LABEL,
			       PT.REQUIRES_APPROVAL,PT.ALLOWENDTASK,PT.USETIMEPLANNING
	  FROM PROJECT_TYPE PT
 LEFT JOIN PROJECT_APPROVAL PU ON(PU.PROJECT_TYPE_ID = PT.ID AND PU.TYPE = 'U')
     WHERE PT.ORGANIZATION = :organization
	   $filterMeeting
  GROUP BY PT.ID, PT.USETIMEPLANNING,PT.USEMERGEREQUEST, PT.EXTERNALTYPE, PT.ORGANIZATION, PT.LABEL,
           PT.REQUIRES_APPROVAL,PT.ALLOWENDTASK
  ORDER BY ID

SQL;

        $params = array(
            'organization' => $organization,
        );
        $connection = $this->getEntityManager()->getConnection();
        $projectTypes = $connection->fetchAll($query, $params);
        foreach ($projectTypes as &$projectType) {
            $projectType['PROFILE'] = Utilities::setSelectMultiInQuery($projectType['PROFILE'], true);
        }
        return $projectTypes;
    }

    public function findIssueFramingProjectType($issueStatusId){

        $query = <<<SQL
          SELECT TPT.ID, PRT.LABEL,TPT.PROJECT_TYPE_ID,PRT.LABEL AS LABEL_FORM
            FROM TASK_TYPE_PROJECT_TYPE TPT
            JOIN PROJECT_TYPE PRT ON (PRT.ID = TPT.PROJECT_TYPE_ID)
           WHERE TPT.STATUS = 'A'
             AND TPT.TASK_TYPE_ID = :issueStatusId
       ORDER BY TPT.ID ASC
SQL;
        $params = array(
            'issueStatusId' => $issueStatusId
        );
        $return = $this->getEntityManager()->getConnection()->fetchAll($query, $params);


        return $return;
    }

    public function findProjectTypes($organizationId,$filterString){

        $query = <<<SQL
          SELECT * FROM (
		   SELECT PRT.ID, PRT.LABEL
			 FROM PROJECT_TYPE PRT
			WHERE PRT.ORGANIZATION = :ORGANIZATION_ID)
			WHERE 1 = 1 $filterString
         ORDER BY LABEL
SQL;
        $params = array(
            'ORGANIZATION_ID' => $organizationId,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
    
}